--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;

ENTITY tech_10gbase_r IS
  GENERIC (
    g_technology     : NATURAL := c_tech_select_default;
    g_sim            : BOOLEAN := FALSE;
    g_sim_level      : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels   : NATURAL := 1
  );
  PORT (
    -- MM
    mm_clk            : IN  STD_LOGIC := '0';
    mm_rst            : IN  STD_LOGIC := '0';
    reg_ip_arria10_e3sge3_phy_10gbase_r_24_mosi   : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e3sge3_phy_10gbase_r_24_miso   : OUT t_mem_miso;
    reg_ip_arria10_e1sg_phy_10gbase_r_24_mosi     : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e1sg_phy_10gbase_r_24_miso     : OUT t_mem_miso;
    reg_ip_arria10_e2sg_phy_10gbase_r_24_mosi     : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e2sg_phy_10gbase_r_24_miso     : OUT t_mem_miso;
    -- Transceiver ATX PLL reference clock
    tr_ref_clk_644          : IN  STD_LOGIC;   -- 644.531250 MHz
    
    -- XGMII clocks
    clk_156                 : IN STD_LOGIC;    -- 156.25 MHz
    rst_156                 : IN STD_LOGIC;

    -- XGMII interface
    xgmii_tx_ready_arr      : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);  -- can be used for xon flow control
    xgmii_rx_ready_arr      : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);  -- typically leave not connected
    xgmii_tx_dc_arr         : IN  t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);  -- 72 bit
    xgmii_rx_dc_arr         : OUT t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);  -- 72 bit

    -- PHY serial IO
    tx_serial_arr           : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    rx_serial_arr           : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
END tech_10gbase_r;


ARCHITECTURE str OF tech_10gbase_r IS  

  CONSTANT c_use_technology : BOOLEAN := g_sim = FALSE OR g_sim_level = 0;
  CONSTANT c_use_sim_model  : BOOLEAN := NOT c_use_technology;
  
BEGIN
 
  gen_ip_arria10 : IF c_use_technology=TRUE AND g_technology=c_tech_arria10 GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (tr_ref_clk_644,
              clk_156, rst_156,
              xgmii_tx_ready_arr, xgmii_rx_ready_arr, xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;
      
  gen_ip_arria10_e3sge3 : IF c_use_technology=TRUE AND g_technology=c_tech_arria10_e3sge3 GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10_e3sge3
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (mm_clk, mm_rst,
              reg_ip_arria10_e3sge3_phy_10gbase_r_24_mosi,
              reg_ip_arria10_e3sge3_phy_10gbase_r_24_miso,   
              tr_ref_clk_644,
              clk_156, rst_156,
              xgmii_tx_ready_arr, xgmii_rx_ready_arr, xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;
      
  gen_ip_arria10_e1sg : IF c_use_technology=TRUE AND g_technology=c_tech_arria10_e1sg GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10_e1sg
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (mm_clk, mm_rst,
              reg_ip_arria10_e1sg_phy_10gbase_r_24_mosi,
              reg_ip_arria10_e1sg_phy_10gbase_r_24_miso, 
              tr_ref_clk_644,
              clk_156, rst_156,
              xgmii_tx_ready_arr, xgmii_rx_ready_arr, xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;

  gen_ip_arria10_e2sg : IF c_use_technology=TRUE AND g_technology=c_tech_arria10_e2sg GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10_e2sg
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (mm_clk, mm_rst,
              reg_ip_arria10_e2sg_phy_10gbase_r_24_mosi,
              reg_ip_arria10_e2sg_phy_10gbase_r_24_miso, 
              tr_ref_clk_644,
              clk_156, rst_156,
              xgmii_tx_ready_arr, xgmii_rx_ready_arr, xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;
      
  gem_sim_10gbase_r : IF c_use_sim_model=TRUE GENERATE
    u0 : ENTITY work.sim_10gbase_r
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (tr_ref_clk_644,
              clk_156, rst_156,
              xgmii_tx_ready_arr, xgmii_rx_ready_arr, xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;
  
END str;

