-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: IP components declarations for various devices that get wrapped by the tech components

LIBRARY IEEE, technology_lib, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

PACKAGE tech_eth_10g_component_pkg IS


  ------------------------------------------------------------------------------
  -- ip_stratixiv
  ------------------------------------------------------------------------------
  COMPONENT ip_stratixiv_eth_10g IS
  GENERIC (
    g_sim                     : BOOLEAN := FALSE;
    g_sim_level               : NATURAL := 0;     -- 0 = use XAUI IP; 1 = use fast serdes model
    g_nof_channels            : NATURAL := 1;
    g_direction               : STRING := "TX_RX";  -- "TX_RX", "TX_ONLY", "RX_ONLY"
    g_pre_header_padding      : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_156   : IN  STD_LOGIC;   -- 156.25 MHz for XAUI
    tr_ref_rst_156   : IN  STD_LOGIC;
    
    -- Calibration & reconfig clock
    cal_rec_clk      : IN  STD_LOGIC;
    
    -- Data clocks
    tx_clk_arr_in    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);  -- 156.25 MHz, externally connect tr_ref_clk_156 to tx_clk_arr_in or connect rx_clk_arr to tx_clk_arr_in
    tx_rst_arr_out   : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    rx_clk_arr_out   : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    rx_clk_arr_in    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);  -- externally connect to rx_clk_arr_out to avoid clock delta-delay
    rx_rst_arr_out   : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    
    -- MM
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;
    
    mac_mosi         : IN  t_mem_mosi;                      -- MAG_10G (CSR)
    mac_miso         : OUT t_mem_miso; 
    
    xaui_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;    -- XAUI control
    xaui_miso        : OUT t_mem_miso;
    
    -- ST
    tx_snk_in_arr    : IN  t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ 156 MHz tx_clk_arr_in
    tx_snk_out_arr   : OUT t_dp_siso_arr(g_nof_channels-1 DOWNTO 0); 
    
    rx_src_out_arr   : OUT t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ 156 MHz rx_clk_arr_in
    rx_src_in_arr    : IN  t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
    
    -- XAUI serial IO
    xaui_tx_arr      : OUT t_xaui_arr(g_nof_channels-1 DOWNTO 0);
    xaui_rx_arr      : IN  t_xaui_arr(g_nof_channels-1 DOWNTO 0)
  );
  END COMPONENT;
  
  
  ------------------------------------------------------------------------------
  -- ip_arria10
  ------------------------------------------------------------------------------
  
  COMPONENT ip_arria10_eth_10g IS
  GENERIC (
    g_sim                 : BOOLEAN := FALSE;
    g_sim_level           : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels        : NATURAL := 1;
    g_direction           : STRING := "TX_RX";  -- "TX_RX", "TX_ONLY", "RX_ONLY"
    g_pre_header_padding  : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   : IN  STD_LOGIC := '0';   -- 644.531250 MHz for 10GBASE-R
    
    -- Data clocks
    clk_312          : IN  STD_LOGIC := '0';
    clk_156          : IN  STD_LOGIC := '0';
    rst_156          : IN  STD_LOGIC := '0';
    
    -- MM
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;
    
    mac_mosi         : IN  t_mem_mosi;         -- MAG_10G (CSR)
    mac_miso         : OUT t_mem_miso; 
    
    reg_eth10g_mosi  : IN  t_mem_mosi;         -- ETH10G (link status register)
    reg_eth10g_miso  : OUT t_mem_miso; 
    
    -- ST
    tx_snk_in_arr    : IN  t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    tx_snk_out_arr   : OUT t_dp_siso_arr(g_nof_channels-1 DOWNTO 0); 
    
    rx_src_out_arr   : OUT t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    rx_src_in_arr    : IN  t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
    
    -- Serial
    serial_tx_arr    : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    serial_rx_arr    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
  END COMPONENT;

  ------------------------------------------------------------------------------
  -- ip_arria10_e3sge3
  ------------------------------------------------------------------------------
  COMPONENT ip_arria10_e3sge3_eth_10g IS  
  GENERIC (
    g_sim                 : BOOLEAN := FALSE;
    g_sim_level           : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels        : NATURAL := 1;
    g_direction           : STRING := "TX_RX";  -- "TX_RX", "TX_ONLY", "RX_ONLY"
    g_pre_header_padding  : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   : IN  STD_LOGIC := '0';   -- 644.531250 MHz for 10GBASE-R
    
    -- Data clocks
    clk_312          : IN  STD_LOGIC := '0';
    clk_156          : IN  STD_LOGIC := '0';
    rst_156          : IN  STD_LOGIC := '0';
    
    -- MM
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;
    
    mac_mosi         : IN  t_mem_mosi;         -- MAG_10G (CSR)
    mac_miso         : OUT t_mem_miso; 
    
    reg_eth10g_mosi  : IN  t_mem_mosi;         -- ETH10G (link status register)
    reg_eth10g_miso  : OUT t_mem_miso;

    reg_ip_arria10_e3sge3_phy_10gbase_r_24_mosi   : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e3sge3_phy_10gbase_r_24_miso   : OUT t_mem_miso;
    
    -- ST
    tx_snk_in_arr    : IN  t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    tx_snk_out_arr   : OUT t_dp_siso_arr(g_nof_channels-1 DOWNTO 0); 
    
    rx_src_out_arr   : OUT t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    rx_src_in_arr    : IN  t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
    
    -- Serial
    serial_tx_arr    : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    serial_rx_arr    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
  END COMPONENT;  

  ------------------------------------------------------------------------------
  -- ip_arria10_e1sg
  ------------------------------------------------------------------------------
  
  COMPONENT ip_arria10_e1sg_eth_10g IS
  GENERIC (
    g_sim                 : BOOLEAN := FALSE;
    g_sim_level           : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels        : NATURAL := 1;
    g_direction           : STRING := "TX_RX";  -- "TX_RX", "TX_ONLY", "RX_ONLY"
    g_use_loopback        : BOOLEAN := FALSE;
    g_pre_header_padding  : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   : IN  STD_LOGIC := '0';   -- 644.531250 MHz for 10GBASE-R
    
    -- Data clocks
    clk_312          : IN  STD_LOGIC := '0';
    clk_156          : IN  STD_LOGIC := '0';
    rst_156          : IN  STD_LOGIC := '0';
    
    -- MM
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;
    
    mac_mosi         : IN  t_mem_mosi;         -- MAG_10G (CSR)
    mac_miso         : OUT t_mem_miso; 
    
    reg_eth10g_mosi  : IN  t_mem_mosi;         -- ETH10G (link status register)
    reg_eth10g_miso  : OUT t_mem_miso;
 
    reg_ip_arria10_e1sg_phy_10gbase_r_24_mosi     : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e1sg_phy_10gbase_r_24_miso     : OUT t_mem_miso;    

    -- ST
    tx_snk_in_arr    : IN  t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    tx_snk_out_arr   : OUT t_dp_siso_arr(g_nof_channels-1 DOWNTO 0); 
    
    rx_src_out_arr   : OUT t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    rx_src_in_arr    : IN  t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
    
    -- Serial
    serial_tx_arr    : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    serial_rx_arr    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
  END COMPONENT;

  ------------------------------------------------------------------------------
  -- ip_arria10_e2sg
  ------------------------------------------------------------------------------
  
  COMPONENT ip_arria10_e2sg_eth_10g IS
  GENERIC (
    g_sim                 : BOOLEAN := FALSE;
    g_sim_level           : NATURAL := 0;     -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels        : NATURAL := 1;
    g_direction           : STRING := "TX_RX";  -- "TX_RX", "TX_ONLY", "RX_ONLY"
    g_use_loopback        : BOOLEAN := FALSE;
    g_pre_header_padding  : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   : IN  STD_LOGIC := '0';   -- 644.531250 MHz for 10GBASE-R
    
    -- Data clocks
    clk_312          : IN  STD_LOGIC := '0';
    clk_156          : IN  STD_LOGIC := '0';
    rst_156          : IN  STD_LOGIC := '0';
    
    -- MM
    mm_clk           : IN  STD_LOGIC;
    mm_rst           : IN  STD_LOGIC;
    
    mac_mosi         : IN  t_mem_mosi;         -- MAG_10G (CSR)
    mac_miso         : OUT t_mem_miso; 
    
    reg_eth10g_mosi  : IN  t_mem_mosi;         -- ETH10G (link status register)
    reg_eth10g_miso  : OUT t_mem_miso;
 
    reg_ip_arria10_e2sg_phy_10gbase_r_24_mosi     : IN  t_mem_mosi := c_mem_mosi_rst; 
    reg_ip_arria10_e2sg_phy_10gbase_r_24_miso     : OUT t_mem_miso;    

    -- ST
    tx_snk_in_arr    : IN  t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    tx_snk_out_arr   : OUT t_dp_siso_arr(g_nof_channels-1 DOWNTO 0); 
    
    rx_src_out_arr   : OUT t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);      -- 64 bit data @ clk_156
    rx_src_in_arr    : IN  t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
    
    -- Serial
    serial_tx_arr    : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    serial_rx_arr    : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
  END COMPONENT;

END tech_eth_10g_component_pkg;

PACKAGE BODY tech_eth_10g_component_pkg IS

END tech_eth_10g_component_pkg;
