-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Instantiate FIFO IP with generics
-- Description:
--   Copied component declaration and instance example from generated/fifo_140/sim/ip_arria10_e2sg_fifo_sc_fifo_140_pkqwcbi.vhd 

LIBRARY ieee;
USE ieee.std_logic_1164.all;

LIBRARY technology_lib;
USE technology_lib.technology_pkg.ALL;

LIBRARY altera_mf;
USE altera_mf.all;

ENTITY ip_arria10_e2sg_fifo_sc IS
  GENERIC (
    g_use_eab   : STRING := "ON";
    g_dat_w     : NATURAL := 20;
    g_nof_words : NATURAL := 1024
  );
  PORT (
    aclr    : IN STD_LOGIC ;
    clock   : IN STD_LOGIC ;
    data    : IN STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
    rdreq   : IN STD_LOGIC ;
    wrreq   : IN STD_LOGIC ;
    empty   : OUT STD_LOGIC ;
    full    : OUT STD_LOGIC ;
    q       : OUT STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0) ;
    usedw   : OUT STD_LOGIC_VECTOR (tech_ceil_log2(g_nof_words)-1 DOWNTO 0)
  );
END ip_arria10_e2sg_fifo_sc;


ARCHITECTURE SYN OF ip_arria10_e2sg_fifo_sc IS

  COMPONENT  scfifo
  GENERIC (
      add_ram_output_register  : STRING;
      intended_device_family  : STRING;
      lpm_numwords  : NATURAL;
      lpm_showahead  : STRING;
      lpm_type  : STRING;
      lpm_width  : NATURAL;
      lpm_widthu  : NATURAL;
      overflow_checking  : STRING;
      underflow_checking  : STRING;
      use_eab  : STRING
  );
  PORT (
      aclr   : IN STD_LOGIC;
      clock   : IN STD_LOGIC;
      data   : IN STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
      rdreq   : IN STD_LOGIC;
      wrreq   : IN STD_LOGIC;
      empty   : OUT STD_LOGIC;
      full   : OUT STD_LOGIC;
      q   : OUT STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
      usedw   : OUT STD_LOGIC_VECTOR (tech_ceil_log2(g_nof_words)-1 DOWNTO 0)
  );
  END COMPONENT;

BEGIN

  u_scfifo : scfifo
  GENERIC MAP (
    add_ram_output_register  => "ON",
    intended_device_family  => "Arria 10",
    lpm_numwords  => g_nof_words,
    lpm_showahead  => "OFF",
    lpm_type  => "scfifo",
    lpm_width  => g_dat_w,
    lpm_widthu  => tech_ceil_log2(g_nof_words),
    overflow_checking  => "ON",
    underflow_checking  => "ON",
    use_eab  => g_use_eab
  )
  PORT MAP (
    aclr => aclr,
    clock => clock,
    data => data,
    rdreq => rdreq,
    wrreq => wrreq,
    empty => empty,
    full => full,
    q => q,
    usedw => usedw
  );

END SYN;
