-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- RadioHDL wrapper

LIBRARY ieee, technology_lib;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;
USE technology_lib.technology_pkg.all;

LIBRARY altera_lnsim;
USE altera_lnsim.altera_lnsim_components.all;

ENTITY ip_arria10_e2sg_ram_crw_crw IS
  GENERIC (
    g_inferred   : BOOLEAN := FALSE;
    g_adr_w      : NATURAL := 5;
    g_dat_w      : NATURAL := 8;
    g_nof_words  : NATURAL := 2**5;
    g_rd_latency : NATURAL := 1;  -- choose 1 or 2
    g_init_file  : STRING  := "UNUSED"
  );
  PORT
  (
    address_a : IN STD_LOGIC_VECTOR (g_adr_w-1 DOWNTO 0);
    address_b : IN STD_LOGIC_VECTOR (g_adr_w-1 DOWNTO 0);
    clk_a     : IN STD_LOGIC  := '1';
    clk_b     : IN STD_LOGIC ;
    data_a    : IN STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
    data_b    : IN STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
    wren_a    : IN STD_LOGIC  := '0';
    wren_b    : IN STD_LOGIC  := '0';
    q_a       : OUT STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
    q_b       : OUT STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0)
  );
END ip_arria10_e2sg_ram_crw_crw;


ARCHITECTURE SYN OF ip_arria10_e2sg_ram_crw_crw IS

  CONSTANT c_outdata_reg_a : STRING := tech_sel_a_b(g_rd_latency=1, "UNREGISTERED", "CLOCK0");
  CONSTANT c_outdata_reg_b : STRING := tech_sel_a_b(g_rd_latency=1, "UNREGISTERED", "CLOCK1");

  COMPONENT altera_syncram
  GENERIC (
          address_reg_b  : string;
          clock_enable_input_a  : string;
          clock_enable_input_b  : string;
          clock_enable_output_a  : string;
          clock_enable_output_b  : string;
          indata_reg_b  : string;
          init_file  : string;
          intended_device_family  : string;
          lpm_type  : string;
          numwords_a  : integer;
          numwords_b  : integer;
          operation_mode  : string;
          outdata_aclr_a  : string;
          outdata_aclr_b  : string;
          outdata_reg_a  : string;
          outdata_reg_b  : string;
          power_up_uninitialized  : string;
          read_during_write_mode_port_a  : string;
          read_during_write_mode_port_b  : string;
          read_during_write_mode_mixed_ports : string;
          widthad_a  : integer;
          widthad_b  : integer;
          width_a  : integer;
          width_b  : integer;
          width_byteena_a  : integer;
          width_byteena_b  : integer
  );
  PORT (
      address_a : in std_logic_vector(g_adr_w-1 downto 0);
      address_b : in std_logic_vector(g_adr_w-1 downto 0);
      clock0 : in std_logic;
      clock1 : in std_logic;
      data_a : in std_logic_vector(g_dat_w-1 downto 0);
      data_b : in std_logic_vector(g_dat_w-1 downto 0);
      wren_a : in std_logic;
      wren_b : in std_logic;
      q_a : out std_logic_vector(g_dat_w - 1 downto 0);
      q_b : out std_logic_vector(g_dat_w - 1 downto 0)
  );
  END COMPONENT;
  
  SIGNAL addr_a : natural range 0 to g_nof_words - 1;
  SIGNAL addr_b : natural range 0 to g_nof_words - 1;
  
  SIGNAL out_a  : STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
  SIGNAL out_b  : STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
  
  SIGNAL reg_a  : STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
  SIGNAL reg_b  : STD_LOGIC_VECTOR (g_dat_w-1 DOWNTO 0);
  
BEGIN

  ASSERT g_rd_latency=1 OR g_rd_latency=2 REPORT "ip_arria10_e2sg_ram_crw_crw : read latency must be 1 (default) or 2" SEVERITY FAILURE;
  
  gen_ip : IF g_inferred=FALSE GENERATE
    -- Copied from ip_arria10_e2sg_ram_crw_crw/ram_2port_2000/sim/ip_arria10_e2sg_ram_crw_crw_ram_2port_2000_zxnv4ey.vhd
    u_altera_syncram : altera_syncram
    GENERIC MAP (
            address_reg_b  => "CLOCK1",
            clock_enable_input_a  => "BYPASS",
            clock_enable_input_b  => "BYPASS",
            clock_enable_output_a  => "BYPASS",
            clock_enable_output_b  => "BYPASS",
            indata_reg_b  => "CLOCK1",
            init_file  => g_init_file,
            intended_device_family  => "Arria 10",
            lpm_type  => "altera_syncram",
            numwords_a  => g_nof_words,
            numwords_b  => g_nof_words,
            operation_mode  => "BIDIR_DUAL_PORT",
            outdata_aclr_a  => "NONE",
            outdata_aclr_b  => "NONE",
            outdata_reg_a  => c_outdata_reg_a,
            outdata_reg_b  => c_outdata_reg_b,
            power_up_uninitialized  => "FALSE",
            read_during_write_mode_port_a  => "NEW_DATA_NO_NBE_READ",
            read_during_write_mode_port_b  => "NEW_DATA_NO_NBE_READ",
            read_during_write_mode_mixed_ports => "OLD_DATA",
            widthad_a  => g_adr_w,
            widthad_b  => g_adr_w,
            width_a  => g_dat_w,
            width_b  => g_dat_w,
            width_byteena_a  => 1,
            width_byteena_b  => 1
    )
    PORT MAP (
        address_a => address_a,
        address_b => address_b,
        clock0 => clk_a,
        clock1 => clk_b,
        data_a => data_a,
        data_b => data_b,
        wren_a => wren_a,
        wren_b => wren_b,
        q_a => q_a,
        q_b => q_b
    );
  END GENERATE;
  
  gen_inferred : IF g_inferred=TRUE GENERATE
    addr_a <= TO_INTEGER(UNSIGNED(address_a));
    addr_b <= TO_INTEGER(UNSIGNED(address_b));
  
    u_mem : entity work.ip_arria10_e2sg_true_dual_port_ram_dual_clock
    generic map (
      DATA_WIDTH => g_dat_w,
      ADDR_WIDTH => g_adr_w
    )
    port map (
      clk_a  => clk_a,
      clk_b  => clk_b,
      addr_a => addr_a,
      addr_b => addr_b,
      data_a => data_a,
      data_b => data_b,
      we_a   => wren_a,
      we_b   => wren_b,
      q_a    => out_a,
      q_b    => out_b
    );
  
    reg_a <= out_a WHEN rising_edge(clk_a);
    reg_b <= out_b WHEN rising_edge(clk_b);
    
    q_a <= out_a WHEN g_rd_latency=1 ELSE reg_a;
    q_b <= out_b WHEN g_rd_latency=1 ELSE reg_b;
  END GENERATE;
    
END SYN;
