-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Test bench for the DDIO in and out simulation models
-- Description:
--               _   _   _   _  
--     inclock  | |_| |_| |_| |_
--     datain    0 1 2 3 4 5 6 7
--     data_h        1   3   5
--     data_l        0   2   4
--     dataout           0 1 2 3 4 5 6 7
--
-- Usage:
--   The tb is self checking (p_verify) and self stopping (tb_end)
--
--   . Load the simulation by right mouse selecting the entity in library work
--   > as 3
--   > run -a

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tb_ip_arria10_e3sge3_ddio_1 IS
END tb_ip_arria10_e3sge3_ddio_1;


ARCHITECTURE tb OF tb_ip_arria10_e3sge3_ddio_1 IS

  CONSTANT c_clk_period : TIME := 10 ns;
  
  SIGNAL tb_end      : STD_LOGIC := '0';
  SIGNAL clk         : STD_LOGIC := '1';
  SIGNAL in_dat      : STD_LOGIC;
  SIGNAL in_data     : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL data_h      : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL data_l      : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL out_data    : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL out_dat     : STD_LOGIC;
  SIGNAL out_dat_exp : STD_LOGIC;
  
BEGIN

  tb_end <= '0', '1' AFTER 100*c_clk_period;

  clk <= NOT clk OR tb_end AFTER c_clk_period/2;
  
  p_in : PROCESS
  BEGIN
    -- 0
    in_dat <= '0';
    WAIT UNTIL falling_edge(clk);
    in_dat <= '0';
    WAIT UNTIL rising_edge(clk);
    -- 1
    in_dat <= '0';
    WAIT UNTIL falling_edge(clk);
    in_dat <= '1';
    WAIT UNTIL rising_edge(clk);
    -- 2
    in_dat <= '1';
    WAIT UNTIL falling_edge(clk);
    in_dat <= '0';
    WAIT UNTIL rising_edge(clk);
    -- 3
    in_dat <= '1';
    WAIT UNTIL falling_edge(clk);
    in_dat <= '1';
    WAIT UNTIL rising_edge(clk);
    -- 2
    in_dat <= '1';
    WAIT UNTIL falling_edge(clk);
    in_dat <= '0';
    WAIT UNTIL rising_edge(clk);
  END PROCESS;

  in_data(0) <= in_dat;
  
  u_ddio_in : ENTITY work.ip_arria10_e3sge3_ddio_in_1
	PORT MAP (
		datain    => in_data,
		inclock   => clk,
		dataout_h => data_h,
		dataout_l => data_l
	);
	
  u_ddio_out : ENTITY work.ip_arria10_e3sge3_ddio_out_1
	PORT MAP (
		dataout  => out_data,
		outclock => clk,
		datain_h => data_h,
		datain_l => data_l
	);
	
	out_dat <= out_data(0);
	
	out_dat_exp <= TRANSPORT in_dat AFTER c_clk_period*1.5 + 1 ps;
	
	p_verify : PROCESS(clk)
	BEGIN
	  IF falling_edge(clk) THEN
	    ASSERT out_dat=out_dat_exp REPORT "tb_ip_arria10_e3sge3_ddio_1: Error, unexpeced data at falling edge";
	  END IF;
	  IF rising_edge(clk) THEN
	    ASSERT out_dat=out_dat_exp REPORT "tb_ip_arria10_e3sge3_ddio_1: Error, unexpeced data at rising edge";
	  END IF;
	END PROCESS;
	
END tb;
