-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Instantiate FIFO IP with generics
-- Description:
--   Copied component declaration and instance example from generated/fifo_140/sim/ip_arria10_e3sge3_fifo_dc_mixed_widths_fifo_140_5csdcfa.vhd

LIBRARY ieee;
USE ieee.std_logic_1164.all;

LIBRARY technology_lib;
USE technology_lib.technology_pkg.ALL;

LIBRARY altera_mf;
USE altera_mf.all;

ENTITY ip_arria10_e3sge3_fifo_dc_mixed_widths IS
  GENERIC (
    g_nof_words : NATURAL := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : NATURAL := 20;
    g_rddat_w   : NATURAL := 10
  );
  PORT (
    aclr    : IN STD_LOGIC  := '0';
    data    : IN STD_LOGIC_VECTOR (g_wrdat_w-1 DOWNTO 0);
    rdclk   : IN STD_LOGIC ;
    rdreq   : IN STD_LOGIC ;
    wrclk   : IN STD_LOGIC ;
    wrreq   : IN STD_LOGIC ;
    q       : OUT STD_LOGIC_VECTOR (g_rddat_w-1 DOWNTO 0);
    rdempty : OUT STD_LOGIC ;
    rdusedw : OUT STD_LOGIC_VECTOR (tech_ceil_log2(g_nof_words*g_wrdat_w/g_rddat_w)-1 DOWNTO 0);
    wrfull  : OUT STD_LOGIC ;
    wrusedw : OUT STD_LOGIC_VECTOR (tech_ceil_log2(g_nof_words)-1 DOWNTO 0)
  );
END ip_arria10_e3sge3_fifo_dc_mixed_widths;


ARCHITECTURE SYN OF ip_arria10_e3sge3_fifo_dc_mixed_widths IS

  COMPONENT  dcfifo_mixed_widths
  GENERIC (
    intended_device_family  : STRING;
    lpm_numwords  : NATURAL;
    lpm_showahead  : STRING;
    lpm_type  : STRING;
    lpm_width  : NATURAL;
    lpm_widthu  : NATURAL;
    lpm_widthu_r  : NATURAL;
    lpm_width_r  : NATURAL;
    overflow_checking  : STRING;
    rdsync_delaypipe  : NATURAL;
    read_aclr_synch  : STRING;
    underflow_checking  : STRING;
    use_eab  : STRING;
    write_aclr_synch  : STRING;
    wrsync_delaypipe  : NATURAL
  );
  PORT (
    aclr   : IN STD_LOGIC;
    data   : IN STD_LOGIC_VECTOR (data'RANGE);
    rdclk   : IN STD_LOGIC;
    rdreq   : IN STD_LOGIC;
    wrclk   : IN STD_LOGIC;
    wrreq   : IN STD_LOGIC;
    q   : OUT STD_LOGIC_VECTOR (q'RANGE);
    rdempty   : OUT STD_LOGIC;
    rdusedw   : OUT STD_LOGIC_VECTOR (rdusedw'RANGE);
    wrfull   : OUT STD_LOGIC;
    wrusedw   : OUT STD_LOGIC_VECTOR (wrusedw'RANGE)
  );
  END COMPONENT;
    
BEGIN

  dcfifo_mixed_widths_component : dcfifo_mixed_widths
  GENERIC MAP (
    intended_device_family  => "Arria 10",
    lpm_numwords  => g_nof_words,
    lpm_showahead  => "OFF",
    lpm_type  => "dcfifo_mixed_widths",
    lpm_width  => g_wrdat_w,
    lpm_widthu  => tech_ceil_log2(g_nof_words),
    lpm_widthu_r  => tech_ceil_log2(g_nof_words*g_wrdat_w/g_rddat_w),
    lpm_width_r  => g_rddat_w,
    overflow_checking  => "ON",
    rdsync_delaypipe  => 5,
    read_aclr_synch  => "OFF",
    underflow_checking  => "ON",
    use_eab  => "ON",
    write_aclr_synch  => "ON",
    wrsync_delaypipe  => 5
  )
  PORT MAP (
    aclr => aclr,
    data => data,
    rdclk => rdclk,
    rdreq => rdreq,
    wrclk => wrclk,
    wrreq => wrreq,
    q => q,
    rdempty => rdempty,
    rdusedw => rdusedw,
    wrfull => wrfull,
    wrusedw => wrusedw
  );

END SYN;
