-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Reusable test bench stimuli for tech_mac_10g receiver
-- Description:

LIBRARY IEEE, technology_lib, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE WORK.tb_tech_mac_10g_pkg.ALL;


ENTITY tb_tech_mac_10_receiver IS
  GENERIC (
    --   g_data_type = c_tb_tech_mac_10g_data_type_symbols  = 0
    --   g_data_type = c_tb_tech_mac_10g_data_type_counter  = 1
    g_data_type             : NATURAL := c_tb_tech_mac_10g_data_type_counter
  );
  PORT (
    mm_init        : IN  STD_LOGIC;
    total_header   : IN  t_network_total_header;
    rx_clk         : IN  STD_LOGIC;
    rx_sosi        : IN  t_dp_sosi;
    rx_siso        : OUT t_dp_siso;
    rx_toggle      : OUT STD_LOGIC  -- toggle after every received packet
  );
END tb_tech_mac_10_receiver;


ARCHITECTURE tb OF tb_tech_mac_10_receiver IS
  SIGNAL i_rx_toggle   : STD_LOGIC := '0';
BEGIN
  rx_toggle <= i_rx_toggle;
  
  p_ff_receiver : PROCESS
  BEGIN
    -- . Avalon ST
    rx_siso <= c_dp_siso_hold;

    WHILE mm_init/='0' LOOP
      WAIT UNTIL rising_edge(rx_clk);
    END LOOP;

    -- Receive forever
    WHILE TRUE LOOP
      proc_tech_mac_10g_rx_packet(total_header, g_data_type, rx_clk, rx_sosi, rx_siso);
      i_rx_toggle <= NOT i_rx_toggle;
    END LOOP;

    WAIT;
  END PROCESS;
END tb;
